@@ -3,6 +3,7 @@ from __future__ import with_statement
 
 import logging
 
+from outflow.core.db import Databases
 from outflow.core.db.alembic.operations import GrantPermissionsOp
 from outflow.core.plugin import Plugin
 from sqlalchemy import engine_from_config, pool
@@ -41,16 +42,16 @@ def create_table(context, revision, op):
 
 
 def get_table_name_list():
-    plugin_name = config.attributes["plugin"]
     db_label = config.attributes["db_label"]
+    plugin_name = config.attributes["plugin"]
+    # db_label = config.attributes["db_label"]
 
     # ensure models submodules are loaded
     Plugin.import_models_submodules(plugin_name)
 
     # get the tables associated with the given database label
-    tables = Plugin.get_models_tables(plugin_name=plugin_name, db_label=db_label)
-    table_names = [table_name for table_name, model_class in tables.items()]
-    return table_names
+    tables = [table.name for table in Databases()[db_label].get_tables()]
+    return tables
 
 
 def include_obj(obj, name, type_, reflected, compare_to):
@@ -89,10 +90,15 @@ def run_migrations_offline():
 
 
 def run_migrations(connection):
+    if connection.dialect.name == "postgresql":
+        include_schemas = True
+    else:
+        include_schemas = False
+
     context.configure(
         connection=connection,
         target_metadata=target_metadata,
-        include_schemas=True,
+        include_schemas=include_schemas,
         compare_type=True,
         process_revision_directives=writer,
         include_object=include_obj,
