@@ -7,14 +7,16 @@ Read values from the file specified by the OUTFLOW_CONFIG_PATH environment varia
 
 import json
 import os
-import pprint
+
+from copy import deepcopy
 from pathlib import Path
 
 import toml
 import yaml
-from outflow.core.generic.functional import LazyObject, empty
+from outflow.core.generic.context_manager import ManagedClass
 from outflow.core.logging import default_config as default_logger_config
-from outflow.core.logging import logger
+
+from .context_manager import PipelineContextManager
 
 ENVIRONMENT_VARIABLE = "OUTFLOW_CONFIG_PATH"
 
@@ -44,20 +46,21 @@ class DictLikeMixin:
     def __contains__(self, key):
         return hasattr(self, key)
 
+    def update(self, values):
+        for key, value in values.items():
+            self[key] = value
 
-class LazyConfig(LazyObject, DictLikeMixin):
-    """
-    A lazy proxy for the pipeline config.
-    """
 
-    def _setup(self):
+class Config(ManagedClass, DictLikeMixin, context_manager=PipelineContextManager):
+    @staticmethod
+    def resolve_lazy_object():
         """
         Load the config file pointed to by the environment variable. This
         is used the first time config is needed.
         """
-        from .settings_management.lazy_settings import settings
+        from outflow.core.pipeline import settings
 
-        logger.debug("Resolving the LazyConfig object")
+        # logger.debug("Resolving the LazyConfig object")
 
         config_filepath = os.environ.get(ENVIRONMENT_VARIABLE)
         if not config_filepath:
@@ -71,45 +74,23 @@ class LazyConfig(LazyObject, DictLikeMixin):
                         config_filepath = filepath
                         break
 
-        self._wrapped = Config(config_filepath)
+        return Config(config_filepath)
 
-    def __repr__(self):
-        # Hardcode the class name as otherwise it yields 'Config'.
-        return "<LazyConfig>"
+    def __init__(self, config_filepath_or_dict):
+        self.name = "__pipeline_config__"
+        self._CONFIG_FILEPATH = None
 
-    def __getattr__(self, name):
-        """Return the value in the config and cache it in self.__dict__."""
-        if self._wrapped is empty:
-            self._setup()
-        val = getattr(self._wrapped, name)
-
-        self.__dict__[name] = val
-        return val
-
-    def __setattr__(self, name, value):
-        """
-        Set the content of config. Clear all cached values if _wrapped changes
-         or clear single values when set.
-        """
-        if name == "_wrapped":
-            self.__dict__.clear()
+        if isinstance(config_filepath_or_dict, dict):
+            self.init_from_dict(config_filepath_or_dict)
         else:
-            self.__dict__.pop(name, None)
-        super().__setattr__(name, value)
+            self.init_from_file(config_filepath_or_dict)
 
-    def __delattr__(self, name):
-        """Delete a config value and clear it from cache if needed."""
-        super().__delattr__(name)
-        self.__dict__.pop(name, None)
+    def init_from_dict(self, config_dict):
+        for key in config_dict:
+            config_value = config_dict[key]
+            self.__setattr__(key, config_value)
 
-    @property
-    def configured(self):
-        """Return True if the settings have already been configured."""
-        return self._wrapped is not empty
-
-
-class Config(DictLikeMixin):
-    def __init__(self, config_filepath):
+    def init_from_file(self, config_filepath):
 
         # store the config filepath in case someone later cares
         self._CONFIG_FILEPATH = config_filepath
@@ -134,14 +115,14 @@ class Config(DictLikeMixin):
         default_config = {
             "logging": default_logger_config,
             "local_mode": False,
-            "use_outflow_schema": False,
         }
 
         for fmt in load_functions:
             try:
+                custom_config = load_functions[fmt](config_file_content)
                 config_dict = {
                     **default_config,
-                    **load_functions[fmt](config_file_content),
+                    **custom_config,
                 }
                 config_format = fmt
             except Exception:
@@ -150,21 +131,66 @@ class Config(DictLikeMixin):
         if config_format is None:
             raise Exception(f"Unsupported config file {config_filepath}")
 
-        for key in config_dict:
-            config_value = config_dict[key]
-            self.__setattr__(key, config_value)
+        config_dict["logging"] = default_logger_config
+
+        default_database_config = {"dialect": "sqlite", "path": "outflow.db"}
+
+        config_dict.setdefault("databases", {})
+
+        if "default" not in config_dict["databases"]:
+            config_dict["databases"]["default"] = default_database_config
+
+        for field in ["formatters", "loggers", "handlers"]:
+            try:
+                config_dict["logging"][field].update(custom_config["logging"][field])
+            except KeyError:
+                continue
+
+        self.init_from_dict(config_dict)
 
     def __repr__(self):
         # skip private keys
-        dct = {
-            key: value
-            for key, value in self.__dict__.items()
-            if not key.startswith("_")
-        }
+        # dct = {
+        #     key: value
+        #     for key, value in self.__dict__.items()
+        #     if not key.startswith("_")
+        # }
         return (
             f'<{self.__class__.__name__} "{self._CONFIG_FILEPATH.as_posix() if self._CONFIG_FILEPATH else "Default"}"\n'
-            f"{pprint.pformat(dct)}>"
+            # f"{pprint.pformat(dct)}>"
         )
 
+    def to_serializable_dict(self):
+        """
+        This sanitize config object for json serialization, remove passwords
+        from database config and remove non serializable attributes
+        """
+        c = deepcopy(self.__dict__)
 
-config = LazyConfig()
+        # cast from PosixPath to string
+        c["_CONFIG_FILEPATH"] = str(c.get("_CONFIG_FILEPATH", None))
+
+        return c
+
+    @staticmethod
+    def sanitize(config_dict):
+        # remove passwords
+        for db_name, db in config_dict.get("databases", {}).items():
+            if db["dialect"] == "postgresql":
+                adm = db["admin"]
+                usr = db["user"]
+                config_dict["databases"][db_name]["admin"] = adm.replace(
+                    adm.split(":")[1], "******"
+                )
+                config_dict["databases"][db_name]["user"] = usr.replace(
+                    usr.split(":")[1], "******"
+                )
+
+        return config_dict
+
+    def __getstate__(self):
+        return self.to_serializable_dict()
+
+    def __setstate__(self, state):
+        vars(self).update(state)
+        self._CONFIG_FILEPATH = Path(self._CONFIG_FILEPATH)
