@@ -1,14 +1,20 @@
 # -*- coding: utf-8 -*-
 import argparse
+import datetime
 import importlib
+import json
 import logging.config
 import os
 import pathlib
 import sys
+import traceback
+from hashlib import sha256
+from uuid import uuid4
 
 from outflow import __version__ as outflow_version
 from outflow.core.logging import logger, set_plugins_loggers_config
-from outflow.core.pipeline.context import PipelineContext
+from outflow.core.pipeline import config, context, settings
+from outflow.core.pipeline.context_manager import PipelineContextManager
 from outflow.core.plugin import Plugin
 
 
@@ -55,11 +61,17 @@ class Pipeline:
             ["-L", "--local-mode"],
             {"help": "Force local mode", "action": "store_true"},
         ),
-        "static_typecheck": (
-            ["--static-typecheck"],
+        # "static_typecheck": (
+        #     ["--static-typecheck"],
+        #     {
+        #         "help": "Use the static typechecker backend to validate the given command",
+        #         "action": "store_true",
+        #     },
+        # ),
+        "backend": (
+            ["--backend", "-b"],
             {
-                "help": "Use the static typechecker backend to validate the given command",
-                "action": "store_true",
+                "help": "Force backend",
             },
         ),
     }
@@ -67,7 +79,7 @@ class Pipeline:
     base_arguments = (
         {  # TODO test underscore (just for information) and switch to caret
             "dry_run": (
-                ["--dry_run"],
+                ["--dry-run"],
                 {
                     "help": "Run the pipeline without the database",
                     "action": "store_true",
@@ -76,7 +88,10 @@ class Pipeline:
         }
     )
 
-    def __init__(
+    def __init__(self, **kwargs):
+        self._kwargs = kwargs
+
+    def _init(
         self,
         *,
         root_directory: str = None,
@@ -92,7 +107,6 @@ class Pipeline:
         """
         from outflow.core.pipeline import config as pipeline_config
 
-        self._context = None
         self._root_command = None
         self.force_dry_run = force_dry_run
 
@@ -135,18 +149,25 @@ class Pipeline:
         logger.debug(f"Config loaded: {pipeline_config}")
 
         pipeline_config["local_mode"] = system_args.local_mode
-        if system_args.static_typecheck:
-            pipeline_config["backend"] = "static_typechecker"
+        if system_args.backend:
+            pipeline_config["backend"] = system_args.backend
+
+        # if system_args.static_typecheck:
+        #     pipeline_config["backend"] = "static_typechecker"
 
         self.load_plugins()
         self.load_backends()
 
     def load_plugins(self):
-        from outflow.core.pipeline import settings
+        from outflow.core.pipeline import context, settings
 
         for plugin_name in settings.PLUGINS:
             Plugin.load(plugin_name)
 
+        # ensure table metadata correspond to the loaded models
+        for model in context._models:
+            model.register()
+
     def load_backends(self):
         from outflow.core.backends.backend import Backend as DefaultBackend
         from outflow.core.pipeline import settings
@@ -158,15 +179,9 @@ class Pipeline:
                 module = importlib.import_module(value)
                 self.backends[key] = module.Backend
                 logger.debug(f"Backend {key} successfully imported")
-            except ImportError:
+            except ImportError as ie:
                 logger.warning(f"Cannot import module {value} for backend {key}")
-
-    @property
-    def context(self):
-        if self._context is None:
-            self._context = PipelineContext(force_dry_run=self.force_dry_run)
-
-        return self._context
+                logger.debug(ie)
 
     @property
     def root_command(self):
@@ -210,35 +225,98 @@ class Pipeline:
 
         return parser.parse_known_args(argv)
 
-    def run(self):
-        """Run the pipeline"""
-        from outflow.core.db import DeclarativeBases
+    def setup_backend(self, backend_name=None):
         from outflow.core.pipeline import config
 
-        if self.display_version:
-            print(f"Outflow version '{outflow_version}'")
-            return 0
+        if not backend_name:
+            backend_name = config.get("backend", "default")
 
-        backend_name = config.get("backend", "default")
         Backend = self.backends.get(backend_name)
         if Backend is None:
             raise ModuleNotFoundError(
                 f"Could not find backend {backend_name}, "
                 "please check that it is correctly imported"
             )
-        backend = Backend(context=self.context)
+        return Backend()
+
+    def run(self):
+        """Run the pipeline"""
+        if self.display_version:
+            print(f"Outflow version '{outflow_version}'")
+            return 0
+
+        # from outflow.core.pipeline import context
+
+        context.force_dry_run = self.force_dry_run
+
+        backend = self.setup_backend()
+
+        command = self.root_command.parse_command_tree(self.command_argv)
+        context.db_untracked = command.db_untracked
+
+        if (
+            command.backend is not None and command.backend != backend.name
+        ):  # FIXME find a way to start ray before parsing command
+            backend.clean()
+            backend = self.setup_backend(command.backend)
+
+        if command.db_untracked:
+            logger.debug(
+                "db_untracked=True passed to command init, this run will not be stored in the database"
+            )
+
+        db_config = self.create_config_in_db()
+        db_run = self.create_run_in_db(db_config)
 
         try:
-            self.result = self.root_command(self.command_argv, backend=backend)
-        except Exception:
+            db_run.start()
+            self.result = backend.run(task_list=command._setup_tasks())
+            db_run.success()
+        except Exception as e:
+            db_run.fail()
+            from outflow.management.models.runtime_exception import RuntimeException
+
+            RuntimeException.create(
+                task=getattr(e, "db_task", None),
+                exception_type=e.__class__.__name__,
+                exception_msg=str(e.args),
+                traceback="".join(traceback.format_tb(e.__traceback__)),
+                time=datetime.datetime.now(),
+            )
+
             raise
         finally:
-            # close database connections and bubble up the exception if any
-            self.context.databases.close()
-            DeclarativeBases.clear_metadata()
+            # close database connections, clear Model metadata and bubble up the exception if any
+            context.databases.close()
+            backend.clean()
 
         return 0
 
+    @staticmethod
+    def create_config_in_db():
+        from outflow.core.pipeline import config, settings
+        from outflow.management.models.configuration import Configuration
+
+        cfg = config.sanitize(config.to_serializable_dict())
+        del cfg["logging"]
+        cfg_json = json.dumps(cfg)
+        settings_json = json.dumps(settings.to_serializable_dict())
+        hash = sha256(bytes(cfg_json + settings_json, encoding="utf-8")).hexdigest()
+        return Configuration.get_or_create(
+            hash=hash,
+            create_method_kwargs={"config": cfg_json, "settings": settings_json},
+        )
+
+    @staticmethod
+    def create_run_in_db(db_config):
+        from outflow.management.models.run import Run
+
+        run_uuid = str(uuid4())
+        db_run = Run.create(uuid=run_uuid, configuration=db_config, state="pending")
+
+        context.db_run = db_run
+        return db_run
+
     @staticmethod
     def get_parent_directory_posix_path(module_path):
         """Return the posix path of the parent directory of the given module
@@ -250,3 +328,22 @@ class Pipeline:
             str: The posix path of the parent directory of the given module
         """
         return pathlib.Path(module_path).parent.resolve().as_posix()
+
+    def __enter__(self):
+        self.pipeline_context_manager = PipelineContextManager().__enter__()
+        self._init(**self._kwargs)
+
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """Exit context, resetting parent_context and context to None"""
+        self.pipeline_context_manager.__exit__(exc_type, exc_val, exc_tb)
+
+
+def get_pipeline_states():
+    states = {
+        "context_state": context.getstate(),
+        "settings_state": settings.getstate(),
+        "config_state": config.getstate(),
+    }
+    return states
