@@ -25,13 +25,16 @@ class Target:
                 # import these modules only if using auto output targets
                 import ast
                 import inspect
+                import textwrap
 
                 class Visitor(ast.NodeVisitor):
                     def visit_Return(self, node: ast.Return):
                         for output in node.value.keys:
                             TaskClass.add_output(cls(name=output.s, *args, **kwargs))
 
-                Visitor().visit(ast.parse(inspect.getsource(TaskClass.run)))
+                Visitor().visit(
+                    ast.parse(textwrap.dedent(inspect.getsource(TaskClass.run)))
+                )
                 return TaskClass
 
             TaskClass.add_output(cls(name=name, *args, **kwargs))
