@@ -8,14 +8,15 @@ from pathlib import Path
 import outflow.core.db.alembic as outflow_alembic
 from alembic import command
 from alembic.config import Config, main
-from outflow.core.command import Command, argument
-from outflow.core.db import DeclarativeBases
+from outflow.core.commands import Command, argument
 from outflow.core.logging import logger
 from outflow.core.plugin import Plugin
-from outflow.core.tasks.task import Task
+from outflow.core.tasks import Task
 from sqlalchemy import UniqueConstraint
-
+from alembic.util.exc import CommandError
 from .management import Management
+from outflow.core.pipeline import settings, context, config
+from ...core.db import Model
 
 
 def generate_alembic_ini_file(alembic_ini_path, alembic_config):
@@ -28,32 +29,29 @@ def generate_alembic_ini_file(alembic_ini_path, alembic_config):
             ini_file.write(f"{key} = {value}\n")
 
 
-def get_alembic_config(db_label, pipeline_context, plugin=None):
-
+def get_alembic_config(db_label, plugin=None):
     version_locations = []
-    for plugin_name in pipeline_context.settings.PLUGINS:
+    for plugin_name in settings.PLUGINS:
         location = Plugin.get_path(plugin_name) / "models" / "versions" / db_label
         if Path(location).is_dir():
             version_locations.append(location.absolute().as_posix())
 
     alembic_path = Path(outflow_alembic.__file__).parent.absolute().as_posix()
 
-    config = {
+    alembic_config = {
         "main_options": {
             "script_location": alembic_path,
             "version_locations": " ".join(version_locations),
             "output_encoding": "utf-8",
         },
         "attributes": {
-            "connection": pipeline_context.databases[db_label].admin_connection,
-            "login_info": pipeline_context.config["databases"][db_label],
-            "Base": DeclarativeBases[db_label],
+            "connection": context.databases[db_label].admin_connection,
+            "login_info": config["databases"][db_label],
+            "Base": Model,
             "db_label": db_label,
         },
         "admin": {
-            "sqlalchemy.url": pipeline_context.databases[db_label]._generate_url(
-                admin=True
-            )
+            "sqlalchemy.url": context.databases[db_label]._generate_url(admin=True)
         },
         "post_write_hooks": {
             "hooks": "black",
@@ -63,9 +61,9 @@ def get_alembic_config(db_label, pipeline_context, plugin=None):
     }
 
     if plugin:
-        config["attributes"]["plugin"] = plugin
+        alembic_config["attributes"]["plugin"] = plugin
 
-    return config
+    return alembic_config
 
 
 @Management.subcommand(invokable=False)
@@ -110,13 +108,13 @@ class ExecuteSQL(Command):
         Executed to clear the database.
         """
 
-        @Task.as_task(with_self=True)
-        def ExecuteCli(self: Task):
+        @Task.as_task
+        def ExecuteCli():
             """
             to execute a script on the outflow database from the command line.
             """
             # get the database objects for the outflow
-            session = self.context.db.session
+            session = context.session
 
             # try closing the session to kill all current transactions
             try:
@@ -127,10 +125,10 @@ class ExecuteSQL(Command):
                 return
 
             # execute the script if present or the command in argument
-            if self.context.args.execute is None:
-                script = self.context.args.input.read()
+            if context.args.execute is None:
+                script = context.args.input.read()
             else:
-                script = self.context.args.execute
+                script = context.args.execute
             logger.debug("running command(s):\n{0}".format(script))
             try:
                 session.execute(script)
@@ -143,17 +141,17 @@ class ExecuteSQL(Command):
         return ExecuteCli()
 
 
-@Db.subcommand(with_task_context=True, allow_extra_args=True)
+@Db.subcommand(allow_extra_args=True)
 @argument("-d", "--db_label", type=str, default="default")
 @argument(
     "command",
     type=str,
     help=""" The command to give to alembic. """,
 )
-def alembic(self: Task):
-    db_label = self.context.args.db_label
+def alembic():
+    db_label = context.args.db_label
 
-    alembic_config = get_alembic_config(db_label, self.context)
+    alembic_config = get_alembic_config(db_label)
 
     with tempfile.TemporaryDirectory() as tmp_dir_path:
         alembic_ini_path = (Path(tmp_dir_path) / "alembic.ini").resolve()
@@ -162,35 +160,28 @@ def alembic(self: Task):
         alembic_args = [
             "-c",
             alembic_ini_path.as_posix(),
-            *self.context.extra_args,
+            *context.extra_args,
         ]
 
         main(argv=alembic_args)
 
 
-def migrate(task, migration_func):
-    revision = task.context.args.revision
+def migrate(migration_func):
+    revision = context.args.revision
 
     if revision is None:
         raise Exception("Missing required argument : revision")
 
-    db_label = task.context.args.db_label
+    db_label = context.args.db_label
     version_locations = list()
 
-    for plugin_name in task.context.settings.PLUGINS:
+    for plugin_name in settings.PLUGINS:
         location = Plugin.get_path(plugin_name) / "models" / "versions" / db_label
         if location.is_dir():
-            for model_file in (Plugin.get_path(plugin_name) / "models").glob(
-                "[!_]*.py"
-            ):
-                db = getattr(
-                    importlib.import_module(plugin_name).models,
-                    model_file.name.split(".")[0],
-                ).Base
-                if db == DeclarativeBases[db_label]:
-                    version_locations.append(location.resolve().as_posix())
+            logger.warning(f"{location}")
+            version_locations.append(location.resolve().as_posix())
 
-    alembic_config_dict = get_alembic_config(db_label, task.context)
+    alembic_config_dict = get_alembic_config(db_label)
     cfg = Config()
 
     for key, value in alembic_config_dict["main_options"].items():
@@ -204,7 +195,7 @@ def migrate(task, migration_func):
     migration_func(cfg, revision)
 
 
-@Db.subcommand(with_task_context=True)
+@Db.subcommand()
 @argument("--db_label", "-d", type=str, default="default")
 @argument(
     "revision",
@@ -218,11 +209,11 @@ def migrate(task, migration_func):
           '+N' N being the number of revisions to execute from the current one.
           """,
 )
-def downgrade(self: Task):
-    migrate(self, command.downgrade)
+def downgrade():
+    migrate(command.downgrade)
 
 
-@Db.subcommand(with_task_context=True)
+@Db.subcommand()
 @argument("--db_label", "-d", type=str, default="default")
 @argument(
     "revision",
@@ -236,15 +227,17 @@ def downgrade(self: Task):
           '+N' N being the number of revisions to execute from the current one.
           """,
 )
-def upgrade(self: Task):
+def upgrade():
     """
     Upgrade the a database to given migration.
     """
 
-    migrate(self, command.upgrade)
+    migrate(command.upgrade)
 
 
-@Db.subcommand(with_task_context=True, allow_extra_args=True)
+@Db.subcommand(
+    allow_extra_args=True,
+)
 @argument(
     "--plugin",
     "-p",
@@ -253,7 +246,7 @@ def upgrade(self: Task):
     help="Name of the plugin for which you want to generate the migrations",
 )
 @argument("--db_label", "-d", type=str, default="default")
-def MakeMigrations(self: Task):
+def MakeMigrations():
     """A task to automatically generate migrations
     'alembic revision --autogenerate' is used to generate migrations.
     You have to manually edit the migration after being generated, because
@@ -262,8 +255,8 @@ def MakeMigrations(self: Task):
     http://alembic.zzzcomputing.com/en/latest/autogenerate.html#what-does-autogenerate-detect-and-what-does-it-not-detect
     """
 
-    plugin = self.context.args.plugin
-    db_label = self.context.args.db_label
+    plugin = context.args.plugin
+    db_label = context.args.db_label
 
     plugin_version_path = Plugin.get_path(plugin) / "models" / "versions" / db_label
 
@@ -281,7 +274,7 @@ def MakeMigrations(self: Task):
     else:
         version_locations = list()
         # get all the previous versions associated with the given base
-        for plugin_name in self.context.settings.PLUGINS:
+        for plugin_name in settings.PLUGINS:
             version_path = (
                 Plugin.get_path(plugin_name) / "models" / "versions" / db_label
             )
@@ -291,7 +284,7 @@ def MakeMigrations(self: Task):
     # check that the plugin is valid
     Plugin.load(plugin)
 
-    alembic_config_dict = get_alembic_config(db_label, self.context, plugin=plugin)
+    alembic_config_dict = get_alembic_config(db_label, plugin=plugin)
     cfg = Config()
 
     for key, value in alembic_config_dict["main_options"].items():
@@ -308,7 +301,7 @@ def MakeMigrations(self: Task):
     # used to pass arguments to alembic revision command
     # http://alembic.zzzcomputing.com/en/latest/api/commands.html#alembic.command.revision
     args = dict()
-    for arg in self.context.extra_args:
+    for arg in context.extra_args:
         try:
             # Only parse keyword of the form (-)-key=val
             if "=" in arg:
@@ -332,12 +325,21 @@ def MakeMigrations(self: Task):
         except IndexError:
             logger.error(f"Unknown argument: {arg}, skipping!")
 
-    command.revision(
-        cfg,
-        autogenerate=True,
-        version_path=plugin_version_path.as_posix(),
-        **args,
-    )
+    try:
+        command.revision(
+            cfg,
+            autogenerate=True,
+            version_path=plugin_version_path.as_posix(),
+            **args,
+        )
+    except CommandError as ce:
+        if "Target database is not up to date." in str(ce):
+            logger.error(
+                "Target database is not up to date, do not forget to run `db upgrade heads` "
+                "before generating more plugin migration"
+            )
+        else:
+            raise ce
 
 
 @Db.subcommand(description="Generate the database documentation of a plugin")
