@@ -3,7 +3,7 @@ import traceback
 
 import ray
 from outflow.core.logging import logger
-from outflow.core.tasks.task_manager import TaskManager
+from outflow.core.tasks import TaskManager
 
 from .base_actor import BaseActor
 
@@ -47,7 +47,7 @@ class MapActor(BaseActor):
         terminating_tasks = list()
 
         for task in workflow_copy:
-            task.context = self.pipeline_context
+            # task.context = self.pipeline_context
             task.parallel_workflow_id = self.index
 
         task_manager = TaskManager()
