@@ -4,46 +4,33 @@ import os
 import platform
 import subprocess
 import sys
+import tempfile
+import threading
 import time
 
 from outflow.core.backends.backend import Backend as DefaultBackend
-from outflow.core.generic.string import as_byte
 from outflow.core.logging import logger
+from outflow.core.logging import LogRecordSocketReceiver
 from outflow.ray.actors import MainActor
+from outflow.core.pipeline import config, get_pipeline_states, context
 
 import ray
 
 
-class StreamToLogger(object):
-    def __init__(self, logger):
-        self.logger = logger
-        self.linebuf = ""
-
-    def write(self, buf):
-        import logging
-
-        for line in buf.rstrip().splitlines():
-            self.logger.log(logging.DEBUG, line.rstrip())
-
-    def flush(self):
-        pass
-
-
 class Backend(DefaultBackend):
-    def __init__(
-        self, context=None, num_cpus=1, resources={"head_node": 1}, *args, **kwargs
-    ):
-        super().__init__(context=context)
+    def __init__(self, num_cpus=1, resources={"head_node": 1}):
+        super().__init__()
         self._job_ids_queue = None
         self.num_nodes = 0
-        self.ray_params = dict()
+        self.head_node_params = dict()
+        self.workers_params = dict()
         self.setup_cluster()
         self.ray_actor = MainActor.options(
             resources=resources, num_cpus=num_cpus
-        ).remote(context=self.pipeline_context, python_path=sys.path)
-
-    def update_pipeline_context_args(self, pipeline_args, extra_args):
-        self.ray_actor.update_pipeline_context_args.remote(pipeline_args, extra_args)
+        ).remote(pipeline_states=get_pipeline_states(), python_path=sys.path)
+        self.tcpserver = LogRecordSocketReceiver()
+        self.name = "ray"
+        self.init_tcp_socket_receiver()
 
     @property
     def job_ids_queue(self):
@@ -54,7 +41,7 @@ class Backend(DefaultBackend):
 
     @staticmethod
     def launch_nodes(
-        ray_params: dict,
+        workers_params: dict,
         num_nodes: int,
         job_ids_q: multiprocessing.Queue,
         stop_event: multiprocessing.Event,
@@ -64,16 +51,17 @@ class Backend(DefaultBackend):
         from simple_slurm import Slurm
 
         # redirect logs from subprocess to logger
-        sys.stdout = StreamToLogger(logger)
+        # sys.stdout = StreamToLogger(logger)
 
         ray_node = Slurm(
-            cpus_per_task=ray_params["num_cpus"],
+            cpus_per_task=workers_params["cpu_per_node"],
+            mem=workers_params["mem_per_node"],
             job_name="ray_node",
         )
 
         for index in range(num_nodes):
             if index > 0:
-                time.sleep(5)
+                time.sleep(3)
 
             if stop_event.is_set():
                 return
@@ -82,13 +70,13 @@ class Backend(DefaultBackend):
 
             sbatch = (
                 "srun {python_path} -m ray.scripts.scripts start --block --address='{redis_address}' "
-                "--num-cpus={num_cpus} "
-                "--redis-password='{_redis_password}' ".format(
+                "--num-cpus={cpu_per_node} "
+                "--redis-password='{_redis_password}'".format(
                     python_path=python_path,
-                    **ray_params
-                    # memory=node_config["memory"],
+                    **workers_params,
                 )
             )
+            logger.warning(f"calling sbatch with : {sbatch}")
 
             job_ids_q.put(ray_node.sbatch(sbatch))
 
@@ -103,7 +91,7 @@ class Backend(DefaultBackend):
 
         # launch ray head server and main worker
 
-        cluster_config = self.pipeline_context.config.get("cluster", {})
+        cluster_config = config.get("cluster", {})
 
         if "mem_per_node" in cluster_config:
             # --- Binary ---
@@ -114,45 +102,64 @@ class Backend(DefaultBackend):
             # --- Decimal ---
             # 1 MB = 1^3 kB = 1 000 000 bytes
 
-            self.ray_params.update({"_memory": as_byte(cluster_config["mem_per_node"])})
+            self.workers_params.update({"mem_per_node": cluster_config["mem_per_node"]})
         if "cpu_per_node" in cluster_config:
-            self.ray_params.update({"num_cpus": cluster_config["cpu_per_node"]})
+            self.workers_params.update({"cpu_per_node": cluster_config["cpu_per_node"]})
 
-        self.ray_params.update(
+        self.workers_params.update(
             {"_redis_password": cluster_config.get("redis_password", "outflow")}
         )
+        self.head_node_params.update(
+            {"_redis_password": self.workers_params["_redis_password"]}
+        )
 
         # FIXME: fix ray to support parallel job on windows
-        if platform.system() == "Windows" or self.pipeline_context.config["local_mode"]:
-            self.ray_params.update({"local_mode": True})
+        if platform.system() == "Windows" or config["local_mode"]:
+            self.head_node_params.update({"local_mode": True})
 
         # needed when plugins are not installed but only in python path
         os.environ["PYTHONPATH"] = ":".join(sys.path)
 
-        ray_info = ray.init(**self.ray_params, resources={"head_node": 1e5})
+        temp_dir = tempfile.mkdtemp(prefix="outflow_ray_")
+        self.head_node_params.update({"_temp_dir": temp_dir})
+        ray_info = ray.init(
+            **self.head_node_params,
+            resources={"head_node": 1e5},
+            object_store_memory=1000000000
+            # log_to_driver=False,
+        )
 
-        self.ray_params.update({"redis_address": ray_info["redis_address"]})
+        self.workers_params.update({"redis_address": ray_info["redis_address"]})
+        context.redis_address = ray_info["redis_address"].split(":")[0]
 
         self.num_nodes = cluster_config.get("num_nodes", 0)
 
+    def init_tcp_socket_receiver(self):
+        logger.debug("About to start TCP server...")
+
+        self.server_thread = threading.Thread(target=self.tcpserver.serve_forever)
+        # Exit the server thread when the main thread terminates
+        self.server_thread.daemon = True
+        self.server_thread.start()
+
     def run(self, *, task_list=[]):
         main_actor_result = self.ray_actor.run.remote(task_list=task_list)
         result = -1
 
         if self.num_nodes > 0:
-            stop_event = multiprocessing.Event()
+            self.stop_event = multiprocessing.Event()
             logger.info(f"Launching {self.num_nodes} ray nodes")
 
-            sbatch_proc = multiprocessing.Process(
+            self.sbatch_proc = multiprocessing.Process(
                 target=self.launch_nodes,
                 args=(
-                    self.ray_params,
+                    self.workers_params,
                     self.num_nodes,
                     self.job_ids_queue,
-                    stop_event,
+                    self.stop_event,
                 ),
             )
-            sbatch_proc.start()
+            self.sbatch_proc.start()
 
         else:
             logger.info(
@@ -162,14 +169,20 @@ class Backend(DefaultBackend):
 
         # main call to the pipeline execution
         result = ray.get(main_actor_result)
+        return result
+
+    def clean(self):
+
+        logger.debug("Cleaning ray backend")
+        self.tcpserver.shutdown()
 
         if self.num_nodes > 0:
-            stop_event.set()
-            sbatch_proc.join()
+            self.stop_event.set()
+            self.sbatch_proc.join()
 
             while not self.job_ids_queue.empty():
                 slurm_id = self.job_ids_queue.get()
                 logger.debug("cancelling slurm id {id}".format(id=slurm_id))
                 subprocess.run(["scancel", str(slurm_id)])
 
-        return result
+        ray.shutdown()
