@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 import sys
 
-from outflow.core.tasks.base_task import BaseTask
+from outflow.core.pipeline import get_pipeline_states
+from outflow.core.tasks import BaseTask
 from outflow.ray.actors import TaskActor
 
 
@@ -13,7 +14,7 @@ class ParallelTask(BaseTask):
         super().__call__()
         self.actor = TaskActor.options(
             resources={"head_node": 1}, num_cpus=self.num_cpus
-        ).remote(context=self.context, python_path=sys.path)
+        ).remote(pipeline_states=get_pipeline_states(), python_path=sys.path)
         self.actor.set_run.remote(self.run)
         actor_result = self.actor.run.remote(
             **kwargs, **self.bind_kwargs, **self.parameterized_kwargs
